@@ -2,8 +2,7 @@
  * mptIO.h
  * -------
  * Purpose: Basic functions for reading/writing binary and endian safe data to/from files/streams.
- * Notes  : This is work-in-progress.
- *          Some useful functions for reading and writing are still missing.
+ * Notes  : Some useful functions for reading and writing are still missing.
  * Authors: Joern Heusipp
  *          OpenMPT Devs
  * The OpenMPT source code is released under the BSD license. Read LICENSE for more details.
@@ -12,20 +11,16 @@
 
 #pragma once
 
+#include "BuildSettings.h"
+
 
-#include "../common/typedefs.h"
-#include "../common/mptTypeTraits.h"
 #include "../common/Endianness.h"
 #include <algorithm>
+#include <array>
 #include <iosfwd>
 #include <limits>
 #include <cstring>
 
-#if defined(MPT_ENABLE_FILEIO_STDIO)
-#include <cstdio>
-#include <stdio.h>
-#endif // MPT_ENABLE_FILEIO_STDIO
-
 
 OPENMPT_NAMESPACE_BEGIN
 
@@ -36,10 +31,10 @@ namespace IO {
 
 typedef int64 Offset;
 
-static const std::size_t BUFFERSIZE_TINY   =    1 * 1024; // on stack usage
-static const std::size_t BUFFERSIZE_SMALL  =    4 * 1024; // on heap
-static const std::size_t BUFFERSIZE_NORMAL =   64 * 1024; // FILE I/O
-static const std::size_t BUFFERSIZE_LARGE  = 1024 * 1024;
+static constexpr std::size_t BUFFERSIZE_TINY   =    1 * 1024; // on stack usage
+static constexpr std::size_t BUFFERSIZE_SMALL  =    4 * 1024; // on heap
+static constexpr std::size_t BUFFERSIZE_NORMAL =   64 * 1024; // FILE I/O
+static constexpr std::size_t BUFFERSIZE_LARGE  = 1024 * 1024;
 
 
 
@@ -76,21 +71,21 @@ bool Flush(std::ostream & f);
 
 
 
-#if defined(MPT_ENABLE_FILEIO_STDIO)
+template <typename Tfile> class WriteBuffer;
+
+template <typename Tfile> bool IsValid(WriteBuffer<Tfile> & f) { return IsValid(f.file()); }
+template <typename Tfile> IO::Offset TellRead(WriteBuffer<Tfile> & f) { return TellRead(f.file()); }
+template <typename Tfile> IO::Offset TellWrite(WriteBuffer<Tfile> & f) { return TellWrite(f.file()); }
+template <typename Tfile> bool SeekBegin(WriteBuffer<Tfile> & f) { return SeekBegin(f.file()); }
+template <typename Tfile> bool SeekEnd(WriteBuffer<Tfile> & f) { return SeekEnd(f.file()); }
+template <typename Tfile> bool SeekAbsolute(WriteBuffer<Tfile> & f, IO::Offset pos) { return SeekAbsolute(f.file(), pos); }
+template <typename Tfile> bool SeekRelative(WriteBuffer<Tfile> & f, IO::Offset off) { return SeekRelative(f.file(), off); }
+template <typename Tfile> IO::Offset ReadRawImpl(WriteBuffer<Tfile> & f, mpt::byte * data, std::size_t size) { return ReadRawImpl(f.file(), data, size); }
+template <typename Tfile> bool WriteRawImpl(WriteBuffer<Tfile> & f, const mpt::byte * data, std::size_t size) { return f.Write(mpt::as_span(data, size)); }
+template <typename Tfile> bool IsEof(WriteBuffer<Tfile> & f) { return IsEof(f.file()); }
+template <typename Tfile> bool Flush(WriteBuffer<Tfile> & f) { return Flush(f.file()); }
 
-bool IsValid(FILE* & f);
-IO::Offset TellRead(FILE* & f);
-IO::Offset TellWrite(FILE* & f);
-bool SeekBegin(FILE* & f);
-bool SeekEnd(FILE* & f);
-bool SeekAbsolute(FILE* & f, IO::Offset pos);
-bool SeekRelative(FILE* & f, IO::Offset off);
-IO::Offset ReadRawImpl(FILE * & f, mpt::byte * data, std::size_t size);
-bool WriteRawImpl(FILE* & f, const mpt::byte * data, std::size_t size);
-bool IsEof(FILE * & f);
-bool Flush(FILE* & f);
 
-#endif // MPT_ENABLE_FILEIO_STDIO
 
 
 
@@ -182,29 +177,65 @@ inline IO::Offset ReadRaw(Tfile & f, Tbyte * data, std::size_t size)
 	return IO::ReadRawImpl(f, mpt::byte_cast<mpt::byte*>(data), size);
 }
 
+template <typename Tbyte, typename Tfile>
+inline IO::Offset ReadRaw(Tfile & f, mpt::span<Tbyte> data)
+{
+	return IO::ReadRawImpl(f, mpt::byte_cast<mpt::byte*>(data.data()), data.size());
+}
+
 template <typename Tbyte, typename Tfile>
 inline bool WriteRaw(Tfile & f, const Tbyte * data, std::size_t size)
 {
 	return IO::WriteRawImpl(f, mpt::byte_cast<const mpt::byte*>(data), size);
 }
 
+template <typename Tbyte, typename Tfile>
+inline bool WriteRaw(Tfile & f, mpt::span<Tbyte> data)
+{
+	return IO::WriteRawImpl(f, mpt::byte_cast<const mpt::byte*>(data.data()), data.size());
+}
+
 template <typename Tbinary, typename Tfile>
 inline bool Read(Tfile & f, Tbinary & v)
 {
-	return IO::ReadRaw(f, mpt::as_raw_memory(v), sizeof(Tbinary)) == sizeof(Tbinary);
+	return IO::ReadRaw(f, mpt::as_raw_memory(v)) == mpt::saturate_cast<mpt::IO::Offset>(mpt::as_raw_memory(v).size());
 }
 
 template <typename Tbinary, typename Tfile>
 inline bool Write(Tfile & f, const Tbinary & v)
 {
-	return IO::WriteRaw(f, mpt::as_raw_memory(v), sizeof(Tbinary));
+	return IO::WriteRaw(f, mpt::as_raw_memory(v));
+}
+
+template <typename Tbinary, typename Tfile>
+inline bool Write(Tfile & f, const std::vector<Tbinary> & v)
+{
+	STATIC_ASSERT(mpt::is_binary_safe<Tbinary>::value);
+	return IO::WriteRaw(f, mpt::as_raw_memory(v));
 }
 
 template <typename T, typename Tfile>
 inline bool WritePartial(Tfile & f, const T & v, size_t size = sizeof(T))
 {
 	MPT_ASSERT(size <= sizeof(T));
-	return IO::WriteRaw(f, mpt::as_raw_memory(v), size);
+	return IO::WriteRaw(f, mpt::as_span(mpt::as_raw_memory(v).data(), size));
+}
+
+template <typename Tfile>
+inline bool ReadByte(Tfile & f, mpt::byte & v)
+{
+	bool result = false;
+	mpt::byte byte = mpt::as_byte(0);
+	const IO::Offset readResult = IO::ReadRaw(f, &byte, sizeof(mpt::byte));
+	if(readResult < 0)
+	{
+		result = false;
+	} else
+	{
+		result = (static_cast<uint64>(readResult) == sizeof(mpt::byte));
+	}
+	v = byte;
+	return result;
 }
 
 template <typename T, typename Tfile>
@@ -212,7 +243,7 @@ inline bool ReadBinaryTruncatedLE(Tfile & f, T & v, std::size_t size)
 {
 	bool result = false;
 	MPT_STATIC_ASSERT(std::numeric_limits<T>::is_integer);
-	mpt::byte bytes[sizeof(T)];
+	uint8 bytes[sizeof(T)];
 	std::memset(bytes, 0, sizeof(T));
 	const IO::Offset readResult = IO::ReadRaw(f, bytes, std::min(size, sizeof(T)));
 	if(readResult < 0)
@@ -222,10 +253,9 @@ inline bool ReadBinaryTruncatedLE(Tfile & f, T & v, std::size_t size)
 	{
 		result = (static_cast<uint64>(readResult) == std::min(size, sizeof(T)));
 	}
-	#ifdef MPT_PLATFORM_BIG_ENDIAN
-		std::reverse(bytes, bytes + sizeof(T));
-	#endif
-	std::memcpy(&v, bytes, sizeof(T));
+	typename mpt::make_le<T>::type val;
+	std::memcpy(&val, bytes, sizeof(T));
+	v = val;
 	return result;
 }
 
@@ -234,7 +264,7 @@ inline bool ReadIntLE(Tfile & f, T & v)
 {
 	bool result = false;
 	STATIC_ASSERT(std::numeric_limits<T>::is_integer);
-	mpt::byte bytes[sizeof(T)];
+	uint8 bytes[sizeof(T)];
 	std::memset(bytes, 0, sizeof(T));
 	const IO::Offset readResult = IO::ReadRaw(f, bytes, sizeof(T));
 	if(readResult < 0)
@@ -244,9 +274,9 @@ inline bool ReadIntLE(Tfile & f, T & v)
 	{
 		result = (static_cast<uint64>(readResult) == sizeof(T));
 	}
-	T val = 0;
+	typename mpt::make_le<T>::type val;
 	std::memcpy(&val, bytes, sizeof(T));
-	v = SwapBytesLE(val);
+	v = val;
 	return result;
 }
 
@@ -255,7 +285,7 @@ inline bool ReadIntBE(Tfile & f, T & v)
 {
 	bool result = false;
 	STATIC_ASSERT(std::numeric_limits<T>::is_integer);
-	mpt::byte bytes[sizeof(T)];
+	uint8 bytes[sizeof(T)];
 	std::memset(bytes, 0, sizeof(T));
 	const IO::Offset readResult = IO::ReadRaw(f, bytes, sizeof(T));
 	if(readResult < 0)
@@ -265,9 +295,9 @@ inline bool ReadIntBE(Tfile & f, T & v)
 	{
 		result = (static_cast<uint64>(readResult) == sizeof(T));
 	}
-	T val = 0;
+	typename mpt::make_be<T>::type val;
 	std::memcpy(&val, bytes, sizeof(T));
-	v = SwapBytesBE(val);
+	v = val;
 	return result;
 }
 
@@ -275,17 +305,17 @@ template <typename Tfile>
 inline bool ReadAdaptiveInt16LE(Tfile & f, uint16 & v)
 {
 	bool result = true;
-	mpt::byte byte = 0;
+	uint8 byte = 0;
 	std::size_t additionalBytes = 0;
 	v = 0;
 	byte = 0;
-	if(!IO::ReadIntLE<mpt::byte>(f, byte)) result = false;
+	if(!IO::ReadIntLE<uint8>(f, byte)) result = false;
 	additionalBytes = (byte & 0x01);
 	v = byte >> 1;
 	for(std::size_t i = 0; i < additionalBytes; ++i)
 	{
 		byte = 0;
-		if(!IO::ReadIntLE<mpt::byte>(f, byte)) result = false;
+		if(!IO::ReadIntLE<uint8>(f, byte)) result = false;
 		v |= (static_cast<uint16>(byte) << (((i+1)*8) - 1));
 	}
 	return result;
@@ -295,17 +325,17 @@ template <typename Tfile>
 inline bool ReadAdaptiveInt32LE(Tfile & f, uint32 & v)
 {
 	bool result = true;
-	mpt::byte byte = 0;
+	uint8 byte = 0;
 	std::size_t additionalBytes = 0;
 	v = 0;
 	byte = 0;
-	if(!IO::ReadIntLE<mpt::byte>(f, byte)) result = false;
+	if(!IO::ReadIntLE<uint8>(f, byte)) result = false;
 	additionalBytes = (byte & 0x03);
 	v = byte >> 2;
 	for(std::size_t i = 0; i < additionalBytes; ++i)
 	{
 		byte = 0;
-		if(!IO::ReadIntLE<mpt::byte>(f, byte)) result = false;
+		if(!IO::ReadIntLE<uint8>(f, byte)) result = false;
 		v |= (static_cast<uint32>(byte) << (((i+1)*8) - 2));
 	}
 	return result;
@@ -315,17 +345,17 @@ template <typename Tfile>
 inline bool ReadAdaptiveInt64LE(Tfile & f, uint64 & v)
 {
 	bool result = true;
-	mpt::byte byte = 0;
+	uint8 byte = 0;
 	std::size_t additionalBytes = 0;
 	v = 0;
 	byte = 0;
-	if(!IO::ReadIntLE<mpt::byte>(f, byte)) result = false;
+	if(!IO::ReadIntLE<uint8>(f, byte)) result = false;
 	additionalBytes = (1 << (byte & 0x03)) - 1;
 	v = byte >> 2;
 	for(std::size_t i = 0; i < additionalBytes; ++i)
 	{
 		byte = 0;
-		if(!IO::ReadIntLE<mpt::byte>(f, byte)) result = false;
+		if(!IO::ReadIntLE<uint8>(f, byte)) result = false;
 		v |= (static_cast<uint64>(byte) << (((i+1)*8) - 2));
 	}
 	return result;
@@ -362,32 +392,32 @@ template <typename T, typename Tfile>
 inline bool WriteIntLE(Tfile & f, const T v)
 {
 	STATIC_ASSERT(std::numeric_limits<T>::is_integer);
-	const T val = SwapBytesLE(v);
-	mpt::byte bytes[sizeof(T)];
-	std::memcpy(bytes, &val, sizeof(T));
-	return IO::WriteRaw(f, bytes, sizeof(T));
+	return IO::Write(f, mpt::as_le(v));
 }
 
 template <typename T, typename Tfile>
 inline bool WriteIntBE(Tfile & f, const T v)
 {
 	STATIC_ASSERT(std::numeric_limits<T>::is_integer);
-	const T val = SwapBytesBE(v);
-	mpt::byte bytes[sizeof(T)];
-	std::memcpy(bytes, &val, sizeof(T));
-	return IO::WriteRaw(f, bytes, sizeof(T));
+	return IO::Write(f, mpt::as_be(v));
 }
 
 template <typename Tfile>
-inline bool WriteAdaptiveInt16LE(Tfile & f, const uint16 v, std::size_t minSize = 0, std::size_t maxSize = 0)
+inline bool WriteAdaptiveInt16LE(Tfile & f, const uint16 v, std::size_t fixedSize = 0)
 {
+	std::size_t minSize = fixedSize;
+	std::size_t maxSize = fixedSize;
 	MPT_ASSERT(minSize == 0 || minSize == 1 || minSize == 2);
 	MPT_ASSERT(maxSize == 0 || maxSize == 1 || maxSize == 2);
 	MPT_ASSERT(maxSize == 0 || maxSize >= minSize);
-	if(v < 0x80 && minSize <= 1 && (1 <= maxSize || maxSize == 0))
+	if(maxSize == 0)
+	{
+		maxSize = 2;
+	}
+	if(v < 0x80 && minSize <= 1 && 1 <= maxSize)
 	{
 		return IO::WriteIntLE<uint8>(f, static_cast<uint8>(v << 1) | 0x00);
-	} else if(v < 0x8000 && minSize <= 2 && (2 <= maxSize || maxSize == 0))
+	} else if(v < 0x8000 && minSize <= 2 && 2 <= maxSize)
 	{
 		return IO::WriteIntLE<uint16>(f, static_cast<uint16>(v << 1) | 0x01);
 	} else
@@ -398,18 +428,24 @@ inline bool WriteAdaptiveInt16LE(Tfile & f, const uint16 v, std::size_t minSize
 }
 
 template <typename Tfile>
-inline bool WriteAdaptiveInt32LE(Tfile & f, const uint32 v, std::size_t minSize = 0, std::size_t maxSize = 0)
+inline bool WriteAdaptiveInt32LE(Tfile & f, const uint32 v, std::size_t fixedSize = 0)
 {
+	std::size_t minSize = fixedSize;
+	std::size_t maxSize = fixedSize;
 	MPT_ASSERT(minSize == 0 || minSize == 1 || minSize == 2 || minSize == 3 || minSize == 4);
 	MPT_ASSERT(maxSize == 0 || maxSize == 1 || maxSize == 2 || maxSize == 3 || maxSize == 4);
 	MPT_ASSERT(maxSize == 0 || maxSize >= minSize);
-	if(v < 0x40 && minSize <= 1 && (1 <= maxSize || maxSize == 0))
+	if(maxSize == 0)
+	{
+		maxSize = 4;
+	}
+	if(v < 0x40 && minSize <= 1 && 1 <= maxSize)
 	{
 		return IO::WriteIntLE<uint8>(f, static_cast<uint8>(v << 2) | 0x00);
-	} else if(v < 0x4000 && minSize <= 2 && (2 <= maxSize || maxSize == 0))
+	} else if(v < 0x4000 && minSize <= 2 && 2 <= maxSize)
 	{
 		return IO::WriteIntLE<uint16>(f, static_cast<uint16>(v << 2) | 0x01);
-	} else if(v < 0x400000 && minSize <= 3 && (3 <= maxSize || maxSize == 0))
+	} else if(v < 0x400000 && minSize <= 3 && 3 <= maxSize)
 	{
 		uint32 value = static_cast<uint32>(v << 2) | 0x02;
 		mpt::byte bytes[3];
@@ -417,7 +453,7 @@ inline bool WriteAdaptiveInt32LE(Tfile & f, const uint32 v, std::size_t minSize
 		bytes[1] = static_cast<mpt::byte>(value >>  8);
 		bytes[2] = static_cast<mpt::byte>(value >> 16);
 		return IO::WriteRaw(f, bytes, 3);
-	} else if(v < 0x40000000 && minSize <= 4 && (4 <= maxSize || maxSize == 0))
+	} else if(v < 0x40000000 && minSize <= 4 && 4 <= maxSize)
 	{
 		return IO::WriteIntLE<uint32>(f, static_cast<uint32>(v << 2) | 0x03);
 	} else
@@ -428,21 +464,27 @@ inline bool WriteAdaptiveInt32LE(Tfile & f, const uint32 v, std::size_t minSize
 }
 
 template <typename Tfile>
-inline bool WriteAdaptiveInt64LE(Tfile & f, const uint64 v, std::size_t minSize = 0, std::size_t maxSize = 0)
+inline bool WriteAdaptiveInt64LE(Tfile & f, const uint64 v, std::size_t fixedSize = 0)
 {
+	std::size_t minSize = fixedSize;
+	std::size_t maxSize = fixedSize;
 	MPT_ASSERT(minSize == 0 || minSize == 1 || minSize == 2 || minSize == 4 || minSize == 8);
 	MPT_ASSERT(maxSize == 0 || maxSize == 1 || maxSize == 2 || maxSize == 4 || maxSize == 8);
 	MPT_ASSERT(maxSize == 0 || maxSize >= minSize);
-	if(v < 0x40 && minSize <= 1 && (1 <= maxSize || maxSize == 0))
+	if(maxSize == 0)
+	{
+		maxSize = 8;
+	}
+	if(v < 0x40 && minSize <= 1 && 1 <= maxSize)
 	{
 		return IO::WriteIntLE<uint8>(f, static_cast<uint8>(v << 2) | 0x00);
-	} else if(v < 0x4000 && minSize <= 2 && (2 <= maxSize || maxSize == 0))
+	} else if(v < 0x4000 && minSize <= 2 && 2 <= maxSize)
 	{
 		return IO::WriteIntLE<uint16>(f, static_cast<uint16>(v << 2) | 0x01);
-	} else if(v < 0x40000000 && minSize <= 4 && (4 <= maxSize || maxSize == 0))
+	} else if(v < 0x40000000 && minSize <= 4 && 4 <= maxSize)
 	{
 		return IO::WriteIntLE<uint32>(f, static_cast<uint32>(v << 2) | 0x02);
-	} else if(v < 0x4000000000000000ull && minSize <= 8 && (8 <= maxSize || maxSize == 0))
+	} else if(v < 0x4000000000000000ull && minSize <= 8 && 8 <= maxSize)
 	{
 		return IO::WriteIntLE<uint64>(f, static_cast<uint64>(v << 2) | 0x03);
 	} else
@@ -523,6 +565,105 @@ inline bool WriteTextLF(Tfile &f, const std::string &s)
 	return mpt::IO::WriteText(f, s) && mpt::IO::WriteTextLF(f);
 }
 
+
+
+// WriteBuffer class that avoids calling to the underlying file writing
+// functions for every operation, which would involve rather slow un-inlinable
+// virtual calls in the iostream and FILE* cases. It is the users responabiliy
+// to call HasWriteError() to check for writeback errors at this buffering
+// level.
+
+template <typename Tfile>
+class WriteBuffer
+{
+private:
+	mpt::byte_span buffer;
+	std::size_t size = 0;
+	Tfile & f;
+	bool writeError = false;
+public:
+	WriteBuffer(const WriteBuffer &) = delete;
+	WriteBuffer & operator=(const WriteBuffer &) = delete;
+public:
+	inline WriteBuffer(Tfile & f_, mpt::byte_span buffer_)
+		: buffer(buffer_)
+		, f(f_)
+	{
+	}
+	inline ~WriteBuffer() noexcept(false)
+	{
+		if(!writeError)
+		{
+			FlushLocal();
+		}
+	}
+public:
+	inline Tfile & file() const
+	{
+		if(IsDirty())
+		{
+			FlushLocal();
+		}
+		return f;
+	}
+public:
+	inline bool HasWriteError() const
+	{
+		return writeError;
+	}
+	inline void ClearError()
+	{
+		writeError = false;
+	}
+	inline bool IsDirty() const
+	{
+		return size > 0;
+	}
+	inline bool IsClean() const
+	{
+		return size == 0;
+	}
+	inline bool IsFull() const
+	{
+		return size == buffer.size();
+	}
+	inline bool Write(mpt::const_byte_span data)
+	{
+		bool result = true;
+		for(std::size_t i = 0; i < data.size(); ++i)
+		{
+			buffer[size] = data[i];
+			size++;
+			if(IsFull())
+			{
+				FlushLocal();
+			}
+		}
+		return result;
+	}
+	inline void FlushLocal()
+	{
+		if(IsClean())
+		{
+			return;
+		}
+		try
+		{
+			if(!mpt::IO::WriteRaw(f, mpt::as_span(buffer.data(), size)))
+			{
+				writeError = true;
+			}
+		} catch (const std::exception &)
+		{
+			writeError = true;
+			throw;
+		}
+		size = 0;
+	}
+};
+
+
+
 } // namespace IO
 
 
@@ -536,9 +677,11 @@ class IFileDataContainer {
 public:
 	typedef std::size_t off_t;
 protected:
-	IFileDataContainer() { }
+	IFileDataContainer() = default;
 public:
-	virtual ~IFileDataContainer() { }
+	IFileDataContainer(const IFileDataContainer&) = default;
+	IFileDataContainer & operator=(const IFileDataContainer&) = default;
+	virtual ~IFileDataContainer() = default;
 public:
 	virtual bool IsValid() const = 0;
 	virtual bool HasFastGetLength() const = 0;
@@ -547,6 +690,11 @@ public:
 	virtual off_t GetLength() const = 0;
 	virtual off_t Read(mpt::byte *dst, off_t pos, off_t count) const = 0;
 
+	virtual off_t Read(off_t pos, mpt::byte_span dst) const
+	{
+		return Read(dst.data(), pos, dst.size());
+	}
+
 	virtual bool CanRead(off_t pos, off_t length) const
 	{
 		off_t dataLength = GetLength();
@@ -576,33 +724,32 @@ public:
 class FileDataContainerDummy : public IFileDataContainer {
 public:
 	FileDataContainerDummy() { }
-	virtual ~FileDataContainerDummy() { }
 public:
-	bool IsValid() const
+	bool IsValid() const override
 	{
 		return false;
 	}
 
-	bool HasFastGetLength() const
+	bool HasFastGetLength() const override
 	{
 		return true;
 	}
 
-	bool HasPinnedView() const
+	bool HasPinnedView() const override
 	{
 		return true;
 	}
 
-	const mpt::byte *GetRawData() const
+	const mpt::byte *GetRawData() const override
 	{
 		return nullptr;
 	}
 
-	off_t GetLength() const
+	off_t GetLength() const override
 	{
 		return 0;
 	}
-	off_t Read(mpt::byte * /*dst*/, off_t /*pos*/, off_t /*count*/) const
+	off_t Read(mpt::byte * /*dst*/, off_t /*pos*/, off_t /*count*/) const override
 	{
 		return 0;
 	}
@@ -617,27 +764,28 @@ private:
 	const off_t dataLength;
 public:
 	FileDataContainerWindow(std::shared_ptr<const IFileDataContainer> src, off_t off, off_t len) : data(src), dataOffset(off), dataLength(len) { }
-	virtual ~FileDataContainerWindow() { }
 
-	bool IsValid() const
+	bool IsValid() const override
 	{
 		return data->IsValid();
 	}
-	bool HasFastGetLength() const
+	bool HasFastGetLength() const override
 	{
 		return data->HasFastGetLength();
 	}
-	bool HasPinnedView() const
+	bool HasPinnedView() const override
 	{
 		return data->HasPinnedView();
 	}
-	const mpt::byte *GetRawData() const {
+	const mpt::byte *GetRawData() const override
+	{
 		return data->GetRawData() + dataOffset;
 	}
-	off_t GetLength() const {
+	off_t GetLength() const override
+	{
 		return dataLength;
 	}
-	off_t Read(mpt::byte *dst, off_t pos, off_t count) const
+	off_t Read(mpt::byte *dst, off_t pos, off_t count) const override
 	{
 		if(pos >= dataLength)
 		{
@@ -645,7 +793,8 @@ public:
 		}
 		return data->Read(dst, dataOffset + pos, std::min(count, dataLength - pos));
 	}
-	bool CanRead(off_t pos, off_t length) const {
+	bool CanRead(off_t pos, off_t length) const override
+	{
 		if((pos == dataLength) && (length == 0))
 		{
 			return true;
@@ -656,7 +805,7 @@ public:
 		}
 		return (length <= dataLength - pos);
 	}
-	off_t GetReadableLength(off_t pos, off_t length) const
+	off_t GetReadableLength(off_t pos, off_t length) const override
 	{
 		if(pos >= dataLength)
 		{
@@ -679,7 +828,6 @@ private:
 protected:
 
 	FileDataContainerSeekable(off_t length);
-	virtual ~FileDataContainerSeekable();
 
 private:
 	
@@ -687,12 +835,12 @@ private:
 
 public:
 
-	bool IsValid() const;
-	bool HasFastGetLength() const;
-	bool HasPinnedView() const;
-	const mpt::byte *GetRawData() const;
-	off_t GetLength() const;
-	off_t Read(mpt::byte *dst, off_t pos, off_t count) const;
+	bool IsValid() const override;
+	bool HasFastGetLength() const override;
+	bool HasPinnedView() const override;
+	const mpt::byte *GetRawData() const override;
+	off_t GetLength() const override;
+	off_t Read(mpt::byte *dst, off_t pos, off_t count) const override;
 
 private:
 
@@ -710,14 +858,13 @@ private:
 public:
 
 	FileDataContainerStdStreamSeekable(std::istream *s);
-	virtual ~FileDataContainerStdStreamSeekable();
 
 	static bool IsSeekable(std::istream *stream);
 	static off_t GetLength(std::istream *stream);
 
 private:
 
-	off_t InternalRead(mpt::byte *dst, off_t pos, off_t count) const;
+	off_t InternalRead(mpt::byte *dst, off_t pos, off_t count) const override;
 
 };
 
@@ -733,12 +880,13 @@ private:
 protected:
 
 	FileDataContainerUnseekable();
-	virtual ~FileDataContainerUnseekable();
 
 private:
 
-	static const std::size_t QUANTUM_SIZE = mpt::IO::BUFFERSIZE_SMALL;
-	static const std::size_t BUFFER_SIZE = mpt::IO::BUFFERSIZE_NORMAL;
+	enum : std::size_t {
+		QUANTUM_SIZE = mpt::IO::BUFFERSIZE_SMALL,
+		BUFFER_SIZE = mpt::IO::BUFFERSIZE_NORMAL
+	};
 
 	void EnsureCacheBuffer(std::size_t requiredbuffersize) const;
 	void CacheStream() const;
@@ -750,14 +898,14 @@ private:
 
 public:
 
-	bool IsValid() const;
-	bool HasFastGetLength() const;
-	bool HasPinnedView() const;
-	const mpt::byte *GetRawData() const;
-	off_t GetLength() const;
-	off_t Read(mpt::byte *dst, off_t pos, off_t count) const;
-	bool CanRead(off_t pos, off_t length) const;
-	off_t GetReadableLength(off_t pos, off_t length) const;
+	bool IsValid() const override;
+	bool HasFastGetLength() const override;
+	bool HasPinnedView() const override;
+	const mpt::byte *GetRawData() const override;
+	off_t GetLength() const override;
+	off_t Read(mpt::byte *dst, off_t pos, off_t count) const override;
+	bool CanRead(off_t pos, off_t length) const override;
+	off_t GetReadableLength(off_t pos, off_t length) const override;
 
 private:
 
@@ -776,12 +924,11 @@ private:
 public:
 
 	FileDataContainerStdStream(std::istream *s);
-	virtual ~FileDataContainerStdStream();
 
 private:
 
-	bool InternalEof() const;
-	off_t InternalRead(mpt::byte *dst, off_t count) const;
+	bool InternalEof() const override;
+	off_t InternalRead(mpt::byte *dst, off_t count) const override;
 
 };
 
@@ -791,9 +938,11 @@ private:
 
 struct CallbackStream
 {
-	static const int SeekSet = 0;
-	static const int SeekCur = 1;
-	static const int SeekEnd = 2;
+	enum : int {
+		SeekSet = 0,
+		SeekCur = 1,
+		SeekEnd = 2
+	};
 	void *stream;
 	std::size_t (*read)( void * stream, void * dst, std::size_t bytes );
 	int (*seek)( void * stream, int64 offset, int whence );
@@ -809,9 +958,8 @@ public:
 	static bool IsSeekable(CallbackStream stream);
 	static off_t GetLength(CallbackStream stream);
 	FileDataContainerCallbackStreamSeekable(CallbackStream s);
-	virtual ~FileDataContainerCallbackStreamSeekable();
 private:
-	off_t InternalRead(mpt::byte *dst, off_t pos, off_t count) const;
+	off_t InternalRead(mpt::byte *dst, off_t pos, off_t count) const override;
 };
 
 
@@ -822,10 +970,9 @@ private:
 	mutable bool eof_reached;
 public:
 	FileDataContainerCallbackStream(CallbackStream s);
-	virtual ~FileDataContainerCallbackStream();
 private:
-	bool InternalEof() const;
-	off_t InternalRead(mpt::byte *dst, off_t count) const;
+	bool InternalEof() const override;
+	off_t InternalRead(mpt::byte *dst, off_t count) const override;
 };
 
 
@@ -841,6 +988,12 @@ class FileDataContainerMemory
 #endif
 {
 
+#if defined(MPT_FILEREADER_STD_ISTREAM)
+#define MPT_FILEDATACONTAINERMEMORY_OVERRIDE override
+#else
+#define MPT_FILEDATACONTAINERMEMORY_OVERRIDE
+#endif
+
 #if !defined(MPT_FILEREADER_STD_ISTREAM)
 public:
 	typedef std::size_t off_t;
@@ -854,39 +1007,35 @@ private:
 public:
 	FileDataContainerMemory() : streamData(nullptr), streamLength(0) { }
 	FileDataContainerMemory(mpt::const_byte_span data) : streamData(data.data()), streamLength(data.size()) { }
-#if defined(MPT_FILEREADER_STD_ISTREAM)
-	virtual
-#endif
-		~FileDataContainerMemory() { }
 
 public:
 
-	bool IsValid() const
+	bool IsValid() const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
 	{
 		return streamData != nullptr;
 	}
 
-	bool HasFastGetLength() const
+	bool HasFastGetLength() const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
 	{
 		return true;
 	}
 
-	bool HasPinnedView() const
+	bool HasPinnedView() const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
 	{
 		return true;
 	}
 
-	const mpt::byte *GetRawData() const
+	const mpt::byte *GetRawData() const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
 	{
 		return streamData;
 	}
 
-	off_t GetLength() const
+	off_t GetLength() const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
 	{
 		return streamLength;
 	}
 
-	off_t Read(mpt::byte *dst, off_t pos, off_t count) const
+	off_t Read(mpt::byte *dst, off_t pos, off_t count) const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
 	{
 		if(pos >= streamLength)
 		{
@@ -897,7 +1046,12 @@ public:
 		return avail;
 	}
 
-	bool CanRead(off_t pos, off_t length) const
+	off_t Read(off_t pos, mpt::byte_span dst) const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
+	{
+		return Read(dst.data(), pos, dst.size());
+	}
+
+	bool CanRead(off_t pos, off_t length) const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
 	{
 		if((pos == streamLength) && (length == 0))
 		{
@@ -910,7 +1064,7 @@ public:
 		return (length <= streamLength - pos);
 	}
 
-	off_t GetReadableLength(off_t pos, off_t length) const
+	off_t GetReadableLength(off_t pos, off_t length) const MPT_FILEDATACONTAINERMEMORY_OVERRIDE
 	{
 		if(pos >= streamLength)
 		{
