@@ -133,22 +133,22 @@ const char Ssb::s_EntryID[3] = {'2','2','8'};
 
 
 #ifdef SSB_LOGGING
-static const MPT_UCHAR_TYPE tstrWriteHeader[] = MPT_ULITERAL("Write header with ID = %1\n");
-static const MPT_UCHAR_TYPE tstrWriteProgress[] = MPT_ULITERAL("Wrote entry: {num, id, rpos, size} = {%1, %2, %3, %4}\n");
-static const MPT_UCHAR_TYPE tstrWritingMap[] = MPT_ULITERAL("Writing map to rpos: %1\n");
-static const MPT_UCHAR_TYPE tstrMapEntryWrite[] = MPT_ULITERAL("Writing map entry: id=%1, rpos=%2, size=%3\n");
-static const MPT_UCHAR_TYPE strWriteNote[] = MPT_ULITERAL("Write note: ");
-static const MPT_UCHAR_TYPE tstrEndOfStream[] = MPT_ULITERAL("End of stream(rpos): %1\n");
+static const MPT_UCHAR_TYPE tstrWriteHeader[] = UL_("Write header with ID = %1\n");
+static const MPT_UCHAR_TYPE tstrWriteProgress[] = UL_("Wrote entry: {num, id, rpos, size} = {%1, %2, %3, %4}\n");
+static const MPT_UCHAR_TYPE tstrWritingMap[] = UL_("Writing map to rpos: %1\n");
+static const MPT_UCHAR_TYPE tstrMapEntryWrite[] = UL_("Writing map entry: id=%1, rpos=%2, size=%3\n");
+static const MPT_UCHAR_TYPE strWriteNote[] = UL_("Write note: ");
+static const MPT_UCHAR_TYPE tstrEndOfStream[] = UL_("End of stream(rpos): %1\n");
 
-static const MPT_UCHAR_TYPE tstrReadingHeader[] = MPT_ULITERAL("Read header with expected ID = %1\n");
-static const MPT_UCHAR_TYPE strNoMapInFile[] = MPT_ULITERAL("No map in the file.\n");
-static const MPT_UCHAR_TYPE strIdMismatch[] = MPT_ULITERAL("ID mismatch, terminating read.\n");
-static const MPT_UCHAR_TYPE strIdMatch[] = MPT_ULITERAL("ID match, continuing reading.\n");
-static const MPT_UCHAR_TYPE tstrReadingMap[] = MPT_ULITERAL("Reading map from rpos: %1\n");
-static const MPT_UCHAR_TYPE tstrEndOfMap[] = MPT_ULITERAL("End of map(rpos): %1\n");
-static const MPT_UCHAR_TYPE tstrReadProgress[] = MPT_ULITERAL("Read entry: {num, id, rpos, size, desc} = {%1, %2, %3, %4, %5}\n");
-static const MPT_UCHAR_TYPE tstrNoEntryFound[] = MPT_ULITERAL("No entry with id %1 found.\n");
-static const MPT_UCHAR_TYPE strReadNote[] = MPT_ULITERAL("Read note: ");
+static const MPT_UCHAR_TYPE tstrReadingHeader[] = UL_("Read header with expected ID = %1\n");
+static const MPT_UCHAR_TYPE strNoMapInFile[] = UL_("No map in the file.\n");
+static const MPT_UCHAR_TYPE strIdMismatch[] = UL_("ID mismatch, terminating read.\n");
+static const MPT_UCHAR_TYPE strIdMatch[] = UL_("ID match, continuing reading.\n");
+static const MPT_UCHAR_TYPE tstrReadingMap[] = UL_("Reading map from rpos: %1\n");
+static const MPT_UCHAR_TYPE tstrEndOfMap[] = UL_("End of map(rpos): %1\n");
+static const MPT_UCHAR_TYPE tstrReadProgress[] = UL_("Read entry: {num, id, rpos, size, desc} = {%1, %2, %3, %4, %5}\n");
+static const MPT_UCHAR_TYPE tstrNoEntryFound[] = UL_("No entry with id %1 found.\n");
+static const MPT_UCHAR_TYPE strReadNote[] = UL_("Read note: ");
 #endif
 
 
@@ -164,8 +164,8 @@ Ssb::Ssb()
 }
 
 
-SsbWrite::SsbWrite(std::ostream& oStrm)
-	: m_pOstrm(&oStrm)
+SsbWrite::SsbWrite(std::ostream& os)
+	: oStrm(os)
 	, m_posEntrycount(0)
 	, m_posMapPosField(0)
 {
@@ -173,8 +173,8 @@ SsbWrite::SsbWrite(std::ostream& oStrm)
 }
 
 
-SsbRead::SsbRead(std::istream& iStrm)
-	: m_pIstrm(&iStrm)
+SsbRead::SsbRead(std::istream& is)
+	: iStrm(is)
 	, m_nReadVersion(0)
 	, m_rposMapBegin(0)
 	, m_posMapEnd(0)
@@ -190,13 +190,13 @@ SsbRead::SsbRead(std::istream& iStrm)
 void SsbWrite::AddWriteNote(const SsbStatus s)
 {
 	m_Status |= s;
-	SSB_LOG(mpt::format(MPT_USTRING("%1: 0x%2\n"))(strWriteNote, mpt::ufmt::hex(s)));
+	SSB_LOG(mpt::format(U_("%1: 0x%2\n"))(strWriteNote, mpt::ufmt::hex(s)));
 }
 
 void SsbRead::AddReadNote(const SsbStatus s)
 {
 	m_Status |= s;
-	SSB_LOG(mpt::format(MPT_USTRING("%1: 0x%2\n"))(strReadNote, mpt::ufmt::hex(s)));
+	SSB_LOG(mpt::format(U_("%1: 0x%2\n"))(strReadNote, mpt::ufmt::hex(s)));
 }
 
 void SsbRead::AddReadNote(const ReadEntry* const pRe, const NumType nNum)
@@ -204,10 +204,10 @@ void SsbRead::AddReadNote(const ReadEntry* const pRe, const NumType nNum)
 	m_Status |= SNT_PROGRESS;
 	SSB_LOG(mpt::format(mpt::ustring(tstrReadProgress))(
 				 nNum,
-				 (pRe && pRe->nIdLength < 30 && m_Idarray.size() > 0) ?  ID(&m_Idarray[pRe->nIdpos], pRe->nIdLength).AsString() : MPT_USTRING(""),
+				 (pRe && pRe->nIdLength < 30 && m_Idarray.size() > 0) ?  ID(&m_Idarray[pRe->nIdpos], pRe->nIdLength).AsString() : U_(""),
 				 (pRe) ? pRe->rposStart : 0,
-				 (pRe && pRe->nSize != invalidDatasize) ? mpt::ufmt::val(pRe->nSize) : MPT_USTRING(""),
-				 MPT_USTRING("")));
+				 (pRe && pRe->nSize != invalidDatasize) ? mpt::ufmt::val(pRe->nSize) : U_(""),
+				 U_("")));
 #ifndef SSB_LOGGING
 	MPT_UNREFERENCED_PARAMETER(pRe);
 	MPT_UNREFERENCED_PARAMETER(nNum);
@@ -242,7 +242,7 @@ void SsbWrite::WriteMapItem(const ID &id,
 						const char* pszDesc)
 {
 	SSB_LOG(mpt::format(mpt::ustring(tstrMapEntryWrite))(
-					(id.GetSize() > 0) ? id.AsString() : MPT_USTRING(""),
+					(id.GetSize() > 0) ? id.AsString() : U_(""),
 					rposDataStart,
 					nDatasize));
 
@@ -285,8 +285,6 @@ void SsbWrite::IncrementWriteCounter()
 
 void SsbWrite::BeginWrite(const ID &id, const uint64& nVersion)
 {
-	std::ostream& oStrm = *m_pOstrm;
-
 	SSB_LOG(mpt::format(mpt::ustring(tstrWriteHeader))(id.AsString()));
 
 	ResetWritestatus();
@@ -368,7 +366,7 @@ SsbRead::ReadRv SsbRead::OnReadEntry(const ReadEntry* pE, const ID &id, const Po
 	{
 		ReadEntry e;
 		e.rposStart = static_cast<RposType>(posReadBegin - m_posStart);
-		e.nSize = static_cast<DataSize>(m_pIstrm->tellg() - posReadBegin);
+		e.nSize = static_cast<DataSize>(iStrm.tellg() - posReadBegin);
 		AddReadNote(&e, m_nCounter);
 	}
 	else // Entry not found.
@@ -386,7 +384,7 @@ SsbRead::ReadRv SsbRead::OnReadEntry(const ReadEntry* pE, const ID &id, const Po
 
 void SsbWrite::OnWroteItem(const ID &id, const Postype& posBeforeWrite)
 {
-	const Offtype nRawEntrySize = m_pOstrm->tellp() - posBeforeWrite;
+	const Offtype nRawEntrySize = oStrm.tellp() - posBeforeWrite;
 
 	if (nRawEntrySize < 0 || static_cast<uint64>(nRawEntrySize) > std::numeric_limits<DataSize>::max())
 		{ AddWriteNote(SNW_INSUFFICIENT_DATASIZETYPE); return; }
@@ -402,7 +400,7 @@ void SsbWrite::OnWroteItem(const ID &id, const Postype& posBeforeWrite)
 		if(nEntrySize <= m_nFixedEntrySize)
 		{
 			for(uint32 i = 0; i<m_nFixedEntrySize-nEntrySize; i++)
-				m_pOstrm->put(0);
+				oStrm.put(0);
 			nEntrySize = m_nFixedEntrySize;
 		}
 		else
@@ -418,8 +416,6 @@ void SsbWrite::OnWroteItem(const ID &id, const Postype& posBeforeWrite)
 
 void SsbRead::BeginRead(const ID &id, const uint64& nVersion)
 {
-	std::istream& iStrm = *m_pIstrm;
-
 	SSB_LOG(mpt::format(mpt::ustring(tstrReadingHeader))(id.AsString()));
 
 	ResetReadstatus();
@@ -444,7 +440,7 @@ void SsbRead::BeginRead(const ID &id, const uint64& nVersion)
 	uint8 storedIdLen = 0;
 	Binaryread<uint8>(iStrm, storedIdLen);
 	char storedIdBuf[256];
-	MemsetZero(storedIdBuf);
+	Clear(storedIdBuf);
 	if(storedIdLen > 0)
 	{
 		iStrm.read(storedIdBuf, storedIdLen);
@@ -574,7 +570,6 @@ void SsbRead::BeginRead(const ID &id, const uint64& nVersion)
 
 void SsbRead::CacheMap()
 {
-	std::istream& iStrm = *m_pIstrm;
 	if(GetFlag(RwfRwHasMap) || m_nFixedEntrySize > 0)
 	{
 		iStrm.seekg(m_posStart + m_rposMapBegin);
@@ -649,7 +644,7 @@ void SsbRead::CacheMap()
 
 	SetFlag(RwfRMapCached, true);
 	m_posDataBegin = (m_rposMapBegin == m_rposEndofHdrData) ? m_posMapEnd : m_posStart + Postype(m_rposEndofHdrData);
-	m_pIstrm->seekg(m_posDataBegin);
+	iStrm.seekg(m_posDataBegin);
 
 	// If there are no positions in the map but there are entry sizes, rposStart will
 	// be relative to data start. Now that posDataBegin is known, make them relative to 
@@ -665,12 +660,12 @@ void SsbRead::CacheMap()
 
 const ReadEntry* SsbRead::Find(const ID &id)
 {
-	m_pIstrm->clear();
+	iStrm.clear();
 	if (GetFlag(RwfRMapCached) == false)
 		CacheMap();
 	
 	if (m_nFixedEntrySize > 0 && GetFlag(RwfRMapHasStartpos) == false && GetFlag(RwfRMapHasSize) == false)
-		m_pIstrm->seekg(m_posDataBegin + Postype(m_nFixedEntrySize * m_nCounter));
+		iStrm.seekg(m_posDataBegin + Postype(m_nFixedEntrySize * m_nCounter));
 
 	if (GetFlag(RwfRMapHasId) == true)
 	{
@@ -682,7 +677,7 @@ const ReadEntry* SsbRead::Find(const ID &id)
 			{
 				m_nNextReadHint = (i + 1) % nEntries;
 				if (mapData[i].rposStart != 0)
-					m_pIstrm->seekg(m_posStart + Postype(mapData[i].rposStart));
+					iStrm.seekg(m_posStart + Postype(mapData[i].rposStart));
 				return &mapData[i];
 			}
 		}
@@ -693,7 +688,6 @@ const ReadEntry* SsbRead::Find(const ID &id)
 
 void SsbWrite::FinishWrite()
 {
-	std::ostream& oStrm = *m_pOstrm;
 	const Postype posDataEnd = oStrm.tellp();
 		
 	Postype posMapStart = oStrm.tellp();
@@ -711,7 +705,7 @@ void SsbWrite::FinishWrite()
 	oStrm.seekp(m_posEntrycount);
 
 	// Write a fixed size=2 Adaptive64LE because space for this value has already been reserved berforehand.
-	mpt::IO::WriteAdaptiveInt64LE(oStrm, m_nCounter, 2, 2);
+	mpt::IO::WriteAdaptiveInt64LE(oStrm, m_nCounter, 2);
 
 	if (GetFlag(RwfRwHasMap))
 	{	// Write map start position.
@@ -719,7 +713,7 @@ void SsbWrite::FinishWrite()
 		const uint64 rposMap = posMapStart - m_posStart;
 
 		// Write a fixed size=8 Adaptive64LE because space for this value has already been reserved berforehand.
-		mpt::IO::WriteAdaptiveInt64LE(oStrm, rposMap, 8, 8);
+		mpt::IO::WriteAdaptiveInt64LE(oStrm, rposMap, 8);
 
 	}
 
