@@ -7,9 +7,10 @@
  * The OpenMPT source code is released under the BSD license. Read LICENSE for more details.
  */
 
-
 #pragma once
 
+#include "BuildSettings.h"
+
 #include "Sndfile.h"
 #include "Dither.h"
 #include "../soundbase/SampleFormat.h"
@@ -41,10 +42,9 @@ public:
 	{
 		MPT_ASSERT(SampleFormat(SampleFormatTraits<Tsample>::sampleFormat).IsValid());
 	}
-	virtual ~AudioReadTargetBuffer() { }
 	std::size_t GetRenderedCount() const { return countRendered; }
 public:
-	virtual void DataCallback(int *MixSoundBuffer, std::size_t channels, std::size_t countChunk)
+	void DataCallback(int32 *MixSoundBuffer, std::size_t channels, std::size_t countChunk) override
 	{
 		// Convert to output sample format and optionally perform dithering and clipping if needed
 
@@ -94,7 +94,7 @@ public:
 	{
 		MPT_ASSERT_ALWAYS(sampleFormat.IsValid());
 	}
-	virtual void DataCallback(int *MixSoundBuffer, std::size_t channels, std::size_t countChunk)
+	void DataCallback(int32 *MixSoundBuffer, std::size_t channels, std::size_t countChunk) override
 	{
 		switch(sampleFormat.value)
 		{
@@ -146,17 +146,89 @@ public:
 };
 
 
+class AudioSourceBuffer
+	: public IAudioSource
+{
+private:
+	std::size_t countRendered;
+protected:
+	SampleFormat sampleFormat;
+	const void *inputBuffer;
+public:
+	AudioSourceBuffer(SampleFormat sampleFormat, const void *buffer)
+		: countRendered(0)
+		, sampleFormat(sampleFormat)
+		, inputBuffer(buffer)
+	{
+		MPT_ASSERT(sampleFormat.IsValid());
+	}
+	virtual ~AudioSourceBuffer() { }
+	std::size_t GetRenderedCount() const { return countRendered; }
+private:
+	template<typename Tsample>
+	void Fill(const Tsample *inputBuffer, int32 * const *MixInputBuffers, std::size_t channels, std::size_t countChunk)
+	{
+		for(std::size_t channel = 0; channel < channels; ++channel)
+		{
+			SC::ConvertToFixedPoint<int32, Tsample, MIXING_FRACTIONAL_BITS> conv;
+			for(std::size_t frame = 0; frame < countChunk; ++frame)
+			{
+				MixInputBuffers[channel][frame] = conv(inputBuffer[channel + ((countRendered + frame) * channels)]);
+			}
+		}
+		countRendered += countChunk;
+	}
+public:
+	virtual void FillCallback(int32 * const *MixInputBuffers, std::size_t channels, std::size_t countChunk)
+	{
+		switch(sampleFormat.value)
+		{
+		case SampleFormatUnsigned8:
+			{
+				typedef SampleFormatToType<SampleFormatUnsigned8>::type Tsample;
+				Fill(reinterpret_cast<const Tsample*>(inputBuffer), MixInputBuffers, channels, countChunk);
+			}
+			break;
+		case SampleFormatInt16:
+			{
+				typedef SampleFormatToType<SampleFormatInt16>::type Tsample;
+				Fill(reinterpret_cast<const Tsample*>(inputBuffer), MixInputBuffers, channels, countChunk);
+			}
+			break;
+		case SampleFormatInt24:
+			{
+				typedef SampleFormatToType<SampleFormatInt24>::type Tsample;
+				Fill(reinterpret_cast<const Tsample*>(inputBuffer), MixInputBuffers, channels, countChunk);
+			}
+			break;
+		case SampleFormatInt32:
+			{
+				typedef SampleFormatToType<SampleFormatInt32>::type Tsample;
+				Fill(reinterpret_cast<const Tsample*>(inputBuffer), MixInputBuffers, channels, countChunk);
+			}
+			break;
+		case SampleFormatFloat32:
+			{
+				typedef SampleFormatToType<SampleFormatFloat32>::type Tsample;
+				Fill(reinterpret_cast<const Tsample*>(inputBuffer), MixInputBuffers, channels, countChunk);
+			}
+			break;
+		}
+	}
+};
+
+
 #else // !MODPLUG_TRACKER
 
 
 template<typename Tsample>
-void ApplyGainBeforeConversionIfAppropriate(int *MixSoundBuffer, std::size_t channels, std::size_t countChunk, float gainFactor)
+void ApplyGainBeforeConversionIfAppropriate(int32 *MixSoundBuffer, std::size_t channels, std::size_t countChunk, float gainFactor)
 {
 	// Apply final output gain for non floating point output
-	ApplyGain(MixSoundBuffer, channels, countChunk, Util::Round<int32>(gainFactor * (1<<16)));
+	ApplyGain(MixSoundBuffer, channels, countChunk, mpt::saturate_round<int32>(gainFactor * (1<<16)));
 }
 template<>
-void ApplyGainBeforeConversionIfAppropriate<float>(int * /*MixSoundBuffer*/, std::size_t /*channels*/, std::size_t /*countChunk*/, float /*gainFactor*/)
+void ApplyGainBeforeConversionIfAppropriate<float>(int32 * /*MixSoundBuffer*/, std::size_t /*channels*/, std::size_t /*countChunk*/, float /*gainFactor*/)
 {
 	// nothing
 }
@@ -188,9 +260,8 @@ public:
 	{
 		return;
 	}
-	virtual ~AudioReadTargetGainBuffer() { }
 public:
-	virtual void DataCallback(int *MixSoundBuffer, std::size_t channels, std::size_t countChunk)
+	void DataCallback(int32 *MixSoundBuffer, std::size_t channels, std::size_t countChunk) override
 	{
 		const std::size_t countRendered_ = Tbase::GetRenderedCount();
 
