@@ -21,15 +21,14 @@
 #ifndef MODPLUG_NO_FILESAVE
 #include "../common/mptFileIO.h"
 #endif
-#include <stdexcept>
+#include "BitReader.h"
 
 
 OPENMPT_NAMESPACE_BEGIN
 
 // Sample decompression routines in other source files
 void AMSUnpack(const int8 * const source, size_t sourceSize, void * const dest, const size_t destSize, char packCharacter);
-uint8 MDLReadBits(uint32 &bitbuf, int32 &bitnum, const uint8 *(&ibuf), size_t &bytesLeft, int8 n);
-uintptr_t DMFUnpack(uint8 *psample, const uint8 *ibuf, const uint8 *ibufmax, uint32 maxlen);
+uintptr_t DMFUnpack(FileReader &file, uint8 *psample, uint32 maxlen);
 
 
 // Read a sample from memory
@@ -59,15 +58,9 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		fileSize = restrictedSampleDataView.size();
 	} else
 	{
-		// Only DMF sample compression encoding should fall in this case,
-		MPT_ASSERT(GetEncoding() == DMF);
-		// file is guaranteed by the caller to be ONLY data for this sample,
-		// it is thus efficient to create a view to the whole file object.
-		// See MPT_ASSERT with fileSize below.
-		restrictedSampleDataView = file.GetPinnedRawDataView();
-		sourceBuf = restrictedSampleDataView.data();
-		fileSize = restrictedSampleDataView.size();
+		MPT_ASSERT_NOTREACHED();
 	}
+
 	if(!IsVariableLengthEncoded() && sample.nLength > 0x40000)
 	{
 		// Limit sample length to available bytes in file to avoid excessive memory allocation.
@@ -94,6 +87,19 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 			maxLength /= encodedBytesPerSample;
 		}
 		LimitMax(sample.nLength, mpt::saturate_cast<SmpLength>(maxLength));
+	} else if(GetEncoding() == IT214 || GetEncoding() == IT215 || GetEncoding() == MDL || GetEncoding() == DMF)
+	{
+		// In the best case, IT compression represents each sample point as a single bit.
+		// In practice, there is of course the two-byte header per compressed block and the initial bit width change.
+		// As a result, if we have a file length of n, we know that the sample can be at most n*8 sample points long.
+		// For DMF, there are at least two bits per sample, and for MDL at least 5 (so both are worse than IT).
+		size_t maxLength = fileSize;
+		uint8 maxSamplesPerByte = 8 / GetNumChannels();
+		if(Util::MaxValueOfType(maxLength) / maxSamplesPerByte >= maxLength)
+			maxLength *= maxSamplesPerByte;
+		else
+			maxLength = Util::MaxValueOfType(maxLength);
+		LimitMax(sample.nLength, mpt::saturate_cast<SmpLength>(maxLength));
 	}
 
 	if(sample.nLength < 1)
@@ -126,7 +132,7 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 			LimitMax(readLength, file.BytesLeft());
 
 			const uint8 *inBuf = mpt::byte_cast<const uint8*>(sourceBuf) + sizeof(compressionTable);
-			int8 *outBuf = sample.pSample8;
+			int8 *outBuf = sample.sample8();
 			int8 delta = 0;
 
 			for(size_t i = readLength; i != 0; i--)
@@ -158,7 +164,7 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 			LimitMax(sourceSize, mpt::saturate_cast<uint32>(packedDataView.size()));
 			bytesRead += sourceSize;
 
-			AMSUnpack(reinterpret_cast<const int8 *>(packedDataView.data()), packedDataView.size(), sample.pSample, sample.GetSampleSizeInBytes(), packCharacter);
+			AMSUnpack(reinterpret_cast<const int8 *>(packedDataView.data()), packedDataView.size(), sample.samplev(), sample.GetSampleSizeInBytes(), packCharacter);
 		}
 	} else if(GetEncoding() == PTM8Dto16 && GetChannelFormat() == mono && GetBitDepth() == 16)
 	{
@@ -169,16 +175,8 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		// Huffman MDL compressed samples
 		if(file.CanRead(8) && (fileSize = file.ReadUint32LE()) >= 4)
 		{
-			FileReader chunk = file.ReadChunk(fileSize);
+			BitReader chunk = file.ReadChunk(fileSize);
 			bytesRead = chunk.GetLength() + 4;
-			uint32 bitBuf = chunk.ReadUint32LE();
-			int32 bitNum = 32;
-
-			restrictedSampleDataView = chunk.GetPinnedRawDataView();
-			sourceBuf = restrictedSampleDataView.data();
-
-			const uint8 *inBuf = reinterpret_cast<const uint8*>(sourceBuf);
-			size_t bytesLeft = chunk.BytesLeft();
 
 			uint8 dlt = 0, lowbyte = 0;
 			const bool is16bit = GetBitDepth() == 16;
@@ -189,20 +187,20 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 					uint8 hibyte;
 					if(is16bit)
 					{
-						lowbyte = MDLReadBits(bitBuf, bitNum, inBuf, bytesLeft, 8);
+						lowbyte = static_cast<uint8>(chunk.ReadBits(8));
 					}
-					bool sign = MDLReadBits(bitBuf, bitNum, inBuf, bytesLeft, 1) != 0;
-					if(MDLReadBits(bitBuf, bitNum, inBuf, bytesLeft, 1))
+					bool sign = chunk.ReadBits(1) != 0;
+					if(chunk.ReadBits(1))
 					{
-						hibyte = MDLReadBits(bitBuf, bitNum, inBuf, bytesLeft, 3);
+						hibyte = static_cast<uint8>(chunk.ReadBits(3));
 					} else
 					{
 						hibyte = 8;
-						while(!MDLReadBits(bitBuf, bitNum, inBuf, bytesLeft, 1))
+						while(!chunk.ReadBits(1))
 						{
 							hibyte += 0x10;
 						}
-						hibyte += MDLReadBits(bitBuf, bitNum, inBuf, bytesLeft, 4);
+						hibyte += static_cast<uint8>(chunk.ReadBits(4));
 					}
 					if(sign)
 					{
@@ -211,14 +209,13 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 					dlt += hibyte;
 					if(!is16bit)
 					{
-						sample.pSample8[j] = dlt;
-					}
-					else
+						sample.sample8()[j] = dlt;
+					} else
 					{
-						sample.pSample16[j] = lowbyte | (dlt << 8);
+						sample.sample16()[j] = lowbyte | (dlt << 8);
 					}
 				}
-			} catch(const std::range_error &)
+			} catch(const BitReader::eof &)
 			{
 				// Data is not sufficient to decode the whole sample
 				//AddToLog(LogWarning, "Truncated MDL sample block");
@@ -229,17 +226,7 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		// DMF Huffman compression
 		if(fileSize > 4)
 		{
-			const uint8 *inBuf = mpt::byte_cast<const uint8*>(sourceBuf);
-			const uint8 *inBufMax = inBuf + fileSize;
-			uint8 *outBuf = static_cast<uint8 *>(sample.pSample);
-			bytesRead = DMFUnpack(outBuf, inBuf, inBufMax, sample.GetSampleSizeInBytes());
-
-			// This assertion ensures that, when using variable length samples,
-			// the caller actually provided a trimmed chunk to read the sample data from.
-			// This is required as we cannot know the encoded sample data size upfront
-			// to construct a properly sized pinned view.
-			MPT_ASSERT(bytesRead == fileSize);
-
+			bytesRead = DMFUnpack(file, mpt::byte_cast<uint8 *>(sample.sampleb()), sample.GetSampleSizeInBytes());
 		}
 #ifdef MODPLUG_TRACKER
 	} else if((GetEncoding() == uLaw || GetEncoding() == aLaw) && GetBitDepth() == 16 && (GetChannelFormat() == mono || GetChannelFormat() == stereoInterleaved))
@@ -317,14 +304,14 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 			   944,   912,  1008,   976,   816,   784,   880,   848,
 		};
 
-		const int16 *lut = GetEncoding() == uLaw ? uLawTable : aLawTable;
+		const auto &lut = GetEncoding() == uLaw ? uLawTable : aLawTable;
 
 		SmpLength readLength = sample.nLength * GetNumChannels();
-		LimitMax(readLength, file.BytesLeft());
+		LimitMax(readLength, mpt::saturate_cast<SmpLength>(file.BytesLeft()));
 		bytesRead = readLength;
 
 		const uint8 *inBuf = mpt::byte_cast<const uint8*>(sourceBuf);
-		int16 *outBuf = sample.pSample16;
+		int16 *outBuf = sample.sample16();
 
 		while(readLength--)
 		{
@@ -375,13 +362,14 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 			bytesRead = CopyStereoSplitSample<SC::DecodeUint8>(sample, sourceBuf, fileSize);
 			break;
 		case deltaPCM:		// 8-Bit / Stereo Split / Delta / PCM
+		case MT2:			// same as deltaPCM, but right channel is stored as a difference from the left channel
 			bytesRead = CopyStereoSplitSample<SC::DecodeInt8Delta>(sample, sourceBuf, fileSize);
-			break;
-		case MT2:		// same as deltaPCM, but right channel is stored as a difference from the left channel
-			bytesRead = CopyStereoSplitSample<SC::DecodeInt8Delta>(sample, sourceBuf, fileSize);
-			for(SmpLength i = 0; i < sample.nLength * 2; i += 2)
+			if(GetEncoding() == MT2)
 			{
-				sample.pSample8[i + 1] = static_cast<int8>(static_cast<uint8>(sample.pSample8[i + 1]) + static_cast<uint8>(sample.pSample8[i]));
+				for(int8 *p = sample.sample8(), *pEnd = p + sample.nLength * 2; p < pEnd; p += 2)
+				{
+					p[1] = static_cast<int8>(static_cast<uint8>(p[0]) + static_cast<uint8>(p[1]));
+				}
 			}
 			break;
 		default:
@@ -467,13 +455,14 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 			bytesRead = CopyStereoSplitSample<SC::DecodeInt16<0x8000u, littleEndian16> >(sample, sourceBuf, fileSize);
 			break;
 		case deltaPCM:		// 16-Bit / Stereo Split / Delta / PCM
-			bytesRead = CopyStereoSplitSample<SC::DecodeInt16Delta<littleEndian16> >(sample, sourceBuf, fileSize);
-			break;
 		case MT2:		// same as deltaPCM, but right channel is stored as a difference from the left channel
 			bytesRead = CopyStereoSplitSample<SC::DecodeInt16Delta<littleEndian16> >(sample, sourceBuf, fileSize);
-			for(SmpLength i = 0; i < sample.nLength * 2; i += 2)
+			if(GetEncoding() == MT2)
 			{
-				sample.pSample16[i + 1] = static_cast<int16>(static_cast<uint16>(sample.pSample16[i + 1]) + static_cast<uint16>(sample.pSample16[i]));
+				for(int16 *p = sample.sample16(), *pEnd = p + sample.nLength * 2; p < pEnd; p += 2)
+				{
+					p[1] = static_cast<int16>(static_cast<uint16>(p[0]) + static_cast<uint16>(p[1]));
+				}
 			}
 			break;
 		default:
@@ -733,7 +722,7 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		if(bytesRead && srcPeak != 1.0f)
 		{
 			// Adjust sample volume so we do not affect relative volume of the sample. Normalizing is only done to increase precision.
-			sample.nGlobalVol = Util::Round<uint16>(Clamp(sample.nGlobalVol * srcPeak, 1.0f, 64.0f));
+			sample.nGlobalVol = mpt::saturate_round<uint16>(Clamp(sample.nGlobalVol * srcPeak, 1.0f, 64.0f));
 			sample.uFlags.set(SMP_MODIFIED);
 		}
 	}
@@ -754,7 +743,7 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		if(bytesRead && srcPeak != 1.0)
 		{
 			// Adjust sample volume so we do not affect relative volume of the sample. Normalizing is only done to increase precision.
-			sample.nGlobalVol = Util::Round<uint16>(Clamp(sample.nGlobalVol * srcPeak, 1.0f, 64.0f));
+			sample.nGlobalVol = mpt::saturate_round<uint16>(Clamp(sample.nGlobalVol * srcPeak, 1.0, 64.0));
 			sample.uFlags.set(SMP_MODIFIED);
 		}
 	}
@@ -860,30 +849,36 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 
 
 // Write a sample to file
-size_t SampleIO::WriteSample(std::ostream *f, const ModSample &sample, SmpLength maxSamples) const
+size_t SampleIO::WriteSample(std::ostream &f, const ModSample &sample, SmpLength maxSamples) const
 {
-	if(!sample.HasSampleData()) return 0;
-
-	union
+	if(sample.uFlags[CHN_ADLIB])
 	{
-		int8 buffer8[8192];
-		int16 buffer16[4096];
-	};
-	const void *const pSampleVoid = sample.pSample;
-	const int8 *const pSample8 = sample.pSample8;
-	const int16 *const pSample16 = sample.pSample16;
+		mpt::IO::Write(f, sample.adlib);
+		return sizeof(sample.adlib);
+	}
+	if(!sample.HasSampleData())
+	{
+		return 0;
+	}
+
+	std::array<mpt::byte, mpt::IO::BUFFERSIZE_TINY> writeBuffer;
+	mpt::IO::WriteBuffer<std::ostream> fb{f, mpt::as_span(writeBuffer)};
+
 	SmpLength numSamples = sample.nLength;
 
-	if(maxSamples && numSamples > maxSamples) numSamples = maxSamples;
+	if(maxSamples && numSamples > maxSamples)
+	{
+		numSamples = maxSamples;
+	}
 
-	size_t len = CalculateEncodedSize(numSamples), bufcount = 0;
+	std::size_t len = CalculateEncodedSize(numSamples);
 
 	if(GetBitDepth() == 16 && GetChannelFormat() == mono && GetEndianness() == littleEndian &&
 		(GetEncoding() == signedPCM || GetEncoding() == unsignedPCM || GetEncoding() == deltaPCM))
 	{
 		// 16-bit little-endian mono samples
 		MPT_ASSERT(len == numSamples * 2);
-		if(!f) return len;
+		const int16 *const pSample16 = sample.sample16();
 		const int16 *p = pSample16;
 		int s_old = 0;
 		const int s_ofs = (GetEncoding() == unsignedPCM) ? 0x8000 : 0;
@@ -899,20 +894,13 @@ size_t SampleIO::WriteSample(std::ostream *f, const ModSample &sample, SmpLength
 			}
 			if(GetEncoding() == deltaPCM)
 			{
-				buffer16[bufcount] = SwapBytesLE((int16)(s_new - s_old));
+				mpt::IO::Write(fb, mpt::as_le(static_cast<int16>(s_new - s_old)));
 				s_old = s_new;
 			} else
 			{
-				buffer16[bufcount] = SwapBytesLE((int16)(s_new + s_ofs));
-			}
-			bufcount++;
-			if(bufcount >= mpt::size(buffer16))
-			{
-				mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer16), bufcount * 2);
-				bufcount = 0;
+				mpt::IO::Write(fb, mpt::as_le(static_cast<int16>(s_new + s_ofs)));
 			}
 		}
-		if (bufcount) mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer16), bufcount * 2);
 	}
 
 	else if(GetBitDepth() == 8 && GetChannelFormat() == stereoSplit &&
@@ -920,33 +908,25 @@ size_t SampleIO::WriteSample(std::ostream *f, const ModSample &sample, SmpLength
 	{
 		// 8-bit Stereo samples (not interleaved)
 		MPT_ASSERT(len == numSamples * 2);
-		if(!f) return len;
+		const int8 *const pSample8 = sample.sample8();
 		const int s_ofs = (GetEncoding() == unsignedPCM) ? 0x80 : 0;
 		for (uint32 iCh=0; iCh<2; iCh++)
 		{
 			const int8 *p = pSample8 + iCh;
 			int s_old = 0;
-
-			bufcount = 0;
-			for (uint32 j=0; j<numSamples; j++)
+			for (SmpLength j = 0; j < numSamples; j++)
 			{
 				int s_new = *p;
 				p += 2;
 				if (GetEncoding() == deltaPCM)
 				{
-					buffer8[bufcount++] = (int8)(s_new - s_old);
+					mpt::IO::Write(fb, static_cast<int8>(s_new - s_old));
 					s_old = s_new;
 				} else
 				{
-					buffer8[bufcount++] = (int8)(s_new + s_ofs);
-				}
-				if(bufcount >= mpt::size(buffer8))
-				{
-					mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer8), bufcount);
-					bufcount = 0;
+					mpt::IO::Write(fb, static_cast<int8>(s_new + s_ofs));
 				}
 			}
-			if (bufcount) mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer8), bufcount);
 		}
 	}
 
@@ -955,65 +935,79 @@ size_t SampleIO::WriteSample(std::ostream *f, const ModSample &sample, SmpLength
 	{
 		// 16-bit little-endian Stereo samples (not interleaved)
 		MPT_ASSERT(len == numSamples * 4);
-		if(!f) return len;
+		const int16 *const pSample16 = sample.sample16();
 		const int s_ofs = (GetEncoding() == unsignedPCM) ? 0x8000 : 0;
 		for (uint32 iCh=0; iCh<2; iCh++)
 		{
 			const int16 *p = pSample16 + iCh;
 			int s_old = 0;
-
-			bufcount = 0;
-			for (SmpLength j=0; j<numSamples; j++)
+			for (SmpLength j = 0; j < numSamples; j++)
 			{
 				int s_new = *p;
 				p += 2;
 				if (GetEncoding() == deltaPCM)
 				{
-					buffer16[bufcount] = SwapBytesLE((int16)(s_new - s_old));
+					mpt::IO::Write(fb, mpt::as_le(static_cast<int16>(s_new - s_old)));
 					s_old = s_new;
 				} else
 				{
-					buffer16[bufcount] = SwapBytesLE((int16)(s_new + s_ofs));
-				}
-				bufcount++;
-				if(bufcount >= mpt::size(buffer16))
-				{
-					mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer16), bufcount * 2);
-					bufcount = 0;
+					mpt::IO::Write(fb, mpt::as_le(static_cast<int16>(s_new + s_ofs)));
 				}
 			}
-			if (bufcount) mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer16), bufcount * 2);
 		}
 	}
 
-	else if((GetBitDepth() == 8 || (GetBitDepth() == 16 && GetEndianness() == GetNativeEndianness())) && GetChannelFormat() == stereoInterleaved && GetEncoding() == signedPCM)
+	else if(GetBitDepth() == 8 && GetChannelFormat() == stereoInterleaved && GetEncoding() == signedPCM)
 	{
 		// Stereo signed interleaved
-		if(f) mpt::IO::WriteRaw(*f, mpt::void_cast<const mpt::byte*>(pSampleVoid), len);
+		MPT_ASSERT(len == numSamples * 2);
+		const int8 *const pSample8 = sample.sample8();
+		mpt::IO::WriteRaw(f, reinterpret_cast<const mpt::byte*>(pSample8), len);
+	}
+
+	else if(GetBitDepth() == 16 && GetChannelFormat() == stereoInterleaved && GetEncoding() == signedPCM && GetEndianness() == littleEndian)
+	{
+		// Stereo signed interleaved
+		MPT_ASSERT(len == numSamples * 4);
+		const int16 *const pSample16 = sample.sample16();
+		const int16 *p = pSample16;
+		for(SmpLength j = 0; j < numSamples; j++)
+		{
+			mpt::IO::Write(fb, mpt::as_le(p[0]));
+			mpt::IO::Write(fb, mpt::as_le(p[1]));
+			p += 2;
+		}
+	}
+
+	else if(GetBitDepth() == 16 && GetChannelFormat() == stereoInterleaved && GetEncoding() == signedPCM && GetEndianness() == bigEndian)
+	{
+		// Stereo signed interleaved
+		MPT_ASSERT(len == numSamples * 4);
+		const int16 *const pSample16 = sample.sample16();
+		const int16 *p = pSample16;
+		for(SmpLength j = 0; j < numSamples; j++)
+		{
+			mpt::IO::Write(fb, mpt::as_be(p[0]));
+			mpt::IO::Write(fb, mpt::as_be(p[1]));
+			p += 2;
+		}
 	}
 
 	else if(GetBitDepth() == 8 && GetChannelFormat() == stereoInterleaved && GetEncoding() == unsignedPCM)
 	{
 		// Stereo unsigned interleaved
 		MPT_ASSERT(len == numSamples * 2);
-		if(!f) return len;
-		for(SmpLength j = 0; j < len; j++)
+		const int8 *const pSample8 = sample.sample8();
+		for(SmpLength j = 0; j < numSamples * 2; j++)
 		{
-			buffer8[bufcount] = (int8)((uint8)(pSample8[j]) + 0x80);
-			bufcount++;
-			if(bufcount >= mpt::size(buffer8))
-			{
-				mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer8), bufcount);
-				bufcount = 0;
-			}
+			mpt::IO::Write(fb, static_cast<int8>(static_cast<uint8>(pSample8[j]) + 0x80));
 		}
-		if (bufcount) mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer8), bufcount);
 	}
 
 	else if(GetEncoding() == IT214 || GetEncoding() == IT215)
 	{
 		// IT2.14-encoded samples
-		ITCompression its(sample, GetEncoding() == IT215, f, numSamples);
+		ITCompression its(sample, GetEncoding() == IT215, &f, numSamples);
 		len = its.GetCompressedSize();
 	}
 
@@ -1022,60 +1016,61 @@ size_t SampleIO::WriteSample(std::ostream *f, const ModSample &sample, SmpLength
 	{
 		MPT_ASSERT(GetBitDepth() == 8);
 		MPT_ASSERT(len == numSamples);
-		if(!f) return len;
-		const int8 *p = pSample8;
-		int sinc = sample.GetElementarySampleSize();
-		int s_old = 0;
-		const int s_ofs = (GetEncoding() == unsignedPCM) ? 0x80 : 0;
-		MPT_MAYBE_CONSTANT_IF(mpt::endian_is_little())
+		if(sample.uFlags[CHN_16BIT])
 		{
-			if (sample.uFlags[CHN_16BIT]) p++;
-		}
-
-		for (SmpLength j = 0; j < len; j++)
+			const int16 *const pSample16 = sample.sample16();
+			const int16 *p = pSample16;
+			int s_old = 0;
+			const int s_ofs = (GetEncoding() == unsignedPCM) ? 0x80 : 0;
+			for(SmpLength j = 0; j < numSamples; j++)
+			{
+				int s_new = mpt::rshift_signed(*p, 8);
+				p++;
+				if(sample.uFlags[CHN_STEREO])
+				{
+					s_new = (s_new + (static_cast<int>(*p)) + 1) / 2;
+					p++;
+				}
+				if(GetEncoding() == deltaPCM)
+				{
+					mpt::IO::Write(fb, static_cast<int8>(s_new - s_old));
+					s_old = s_new;
+				} else
+				{
+					mpt::IO::Write(fb, static_cast<int8>(s_new + s_ofs));
+				}
+			}
+		} else
 		{
-			int s_new = (int8)(*p);
-			p += sinc;
-			if (sample.uFlags[CHN_STEREO])
+			const int8 *const pSample8 = sample.sample8();
+			const int8 *p = pSample8;
+			int s_old = 0;
+			const int s_ofs = (GetEncoding() == unsignedPCM) ? 0x80 : 0;
+			for(SmpLength j = 0; j < numSamples; j++)
 			{
-				s_new = (s_new + ((int)*p) + 1) / 2;
-				p += sinc;
-			}
-			if (GetEncoding() == deltaPCM)
-			{
-				buffer8[bufcount++] = (int8)(s_new - s_old);
-				s_old = s_new;
-			} else
-			{
-				buffer8[bufcount++] = (int8)(s_new + s_ofs);
-			}
-			if(bufcount >= mpt::size(buffer8))
-			{
-				mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer8), bufcount);
-				bufcount = 0;
+				int s_new = *p;
+				p++;
+				if(sample.uFlags[CHN_STEREO])
+				{
+					s_new = (s_new + (static_cast<int>(*p)) + 1) / 2;
+					p++;
+				}
+				if(GetEncoding() == deltaPCM)
+				{
+					mpt::IO::Write(fb, static_cast<int8>(s_new - s_old));
+					s_old = s_new;
+				} else
+				{
+					mpt::IO::Write(fb, static_cast<int8>(s_new + s_ofs));
+				}
 			}
 		}
-		if (bufcount) mpt::IO::WriteRaw(*f, reinterpret_cast<mpt::byte*>(buffer8), bufcount);
 	}
+
 	return len;
 }
 
 
-// Write a sample to file
-size_t SampleIO::WriteSample(std::ostream &f, const ModSample &sample, SmpLength maxSamples) const
-{
-	return WriteSample(&f, sample, maxSamples);
-}
-
-
-// Write a sample to file
-size_t SampleIO::WriteSample(FILE *f, const ModSample &sample, SmpLength maxSamples) const
-{
-	mpt::FILE_ostream s(f);
-	return WriteSample(f ? &s : nullptr, sample, maxSamples);
-}
-
-
 #endif // MODPLUG_NO_FILESAVE
 
 
