@@ -10,6 +10,8 @@
 
 #pragma once
 
+#include "BuildSettings.h"
+
 #include <string>
 
 OPENMPT_NAMESPACE_BEGIN
@@ -33,7 +35,7 @@ struct FileTags
 
 	mpt::ustring genre;
 
-	FileTags();
+	void SetEncoder();
 
 };
 
